// Copyright 2022 Google LLC
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     https://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

// Code generated by protoc-gen-go_gapic. DO NOT EDIT.

package publicca

import (
	"context"
	"fmt"
	"math"
	"net/url"
	"time"

	publiccapb "cloud.google.com/go/security/publicca/apiv1beta1/publiccapb"
	gax "github.com/googleapis/gax-go/v2"
	"google.golang.org/api/option"
	"google.golang.org/api/option/internaloption"
	gtransport "google.golang.org/api/transport/grpc"
	"google.golang.org/grpc"
	"google.golang.org/grpc/codes"
	"google.golang.org/grpc/metadata"
)

var newPublicCertificateAuthorityClientHook clientHook

// PublicCertificateAuthorityCallOptions contains the retry settings for each method of PublicCertificateAuthorityClient.
type PublicCertificateAuthorityCallOptions struct {
	CreateExternalAccountKey []gax.CallOption
}

func defaultPublicCertificateAuthorityGRPCClientOptions() []option.ClientOption {
	return []option.ClientOption{
		internaloption.WithDefaultEndpoint("publicca.googleapis.com:443"),
		internaloption.WithDefaultMTLSEndpoint("publicca.mtls.googleapis.com:443"),
		internaloption.WithDefaultAudience("https://publicca.googleapis.com/"),
		internaloption.WithDefaultScopes(DefaultAuthScopes()...),
		internaloption.EnableJwtWithScope(),
		option.WithGRPCDialOption(grpc.WithDefaultCallOptions(
			grpc.MaxCallRecvMsgSize(math.MaxInt32))),
	}
}

func defaultPublicCertificateAuthorityCallOptions() *PublicCertificateAuthorityCallOptions {
	return &PublicCertificateAuthorityCallOptions{
		CreateExternalAccountKey: []gax.CallOption{
			gax.WithRetry(func() gax.Retryer {
				return gax.OnCodes([]codes.Code{
					codes.Unavailable,
				}, gax.Backoff{
					Initial:    100 * time.Millisecond,
					Max:        60000 * time.Millisecond,
					Multiplier: 1.30,
				})
			}),
		},
	}
}

// internalPublicCertificateAuthorityClient is an interface that defines the methods available from Public Certificate Authority API.
type internalPublicCertificateAuthorityClient interface {
	Close() error
	setGoogleClientInfo(...string)
	Connection() *grpc.ClientConn
	CreateExternalAccountKey(context.Context, *publiccapb.CreateExternalAccountKeyRequest, ...gax.CallOption) (*publiccapb.ExternalAccountKey, error)
}

// PublicCertificateAuthorityClient is a client for interacting with Public Certificate Authority API.
// Methods, except Close, may be called concurrently. However, fields must not be modified concurrently with method calls.
//
// Manages the resources required for ACME external account
// binding (at https://tools.ietf.org/html/rfc8555#section-7.3.4) for
// the public certificate authority service.
type PublicCertificateAuthorityClient struct {
	// The internal transport-dependent client.
	internalClient internalPublicCertificateAuthorityClient

	// The call options for this service.
	CallOptions *PublicCertificateAuthorityCallOptions
}

// Wrapper methods routed to the internal client.

// Close closes the connection to the API service. The user should invoke this when
// the client is no longer required.
func (c *PublicCertificateAuthorityClient) Close() error {
	return c.internalClient.Close()
}

// setGoogleClientInfo sets the name and version of the application in
// the `x-goog-api-client` header passed on each request. Intended for
// use by Google-written clients.
func (c *PublicCertificateAuthorityClient) setGoogleClientInfo(keyval ...string) {
	c.internalClient.setGoogleClientInfo(keyval...)
}

// Connection returns a connection to the API service.
//
// Deprecated: Connections are now pooled so this method does not always
// return the same resource.
func (c *PublicCertificateAuthorityClient) Connection() *grpc.ClientConn {
	return c.internalClient.Connection()
}

// CreateExternalAccountKey creates a new ExternalAccountKey bound to the project.
func (c *PublicCertificateAuthorityClient) CreateExternalAccountKey(ctx context.Context, req *publiccapb.CreateExternalAccountKeyRequest, opts ...gax.CallOption) (*publiccapb.ExternalAccountKey, error) {
	return c.internalClient.CreateExternalAccountKey(ctx, req, opts...)
}

// publicCertificateAuthorityGRPCClient is a client for interacting with Public Certificate Authority API over gRPC transport.
//
// Methods, except Close, may be called concurrently. However, fields must not be modified concurrently with method calls.
type publicCertificateAuthorityGRPCClient struct {
	// Connection pool of gRPC connections to the service.
	connPool gtransport.ConnPool

	// flag to opt out of default deadlines via GOOGLE_API_GO_EXPERIMENTAL_DISABLE_DEFAULT_DEADLINE
	disableDeadlines bool

	// Points back to the CallOptions field of the containing PublicCertificateAuthorityClient
	CallOptions **PublicCertificateAuthorityCallOptions

	// The gRPC API client.
	publicCertificateAuthorityClient publiccapb.PublicCertificateAuthorityServiceClient

	// The x-goog-* metadata to be sent with each request.
	xGoogMetadata metadata.MD
}

// NewPublicCertificateAuthorityClient creates a new public certificate authority service client based on gRPC.
// The returned client must be Closed when it is done being used to clean up its underlying connections.
//
// Manages the resources required for ACME external account
// binding (at https://tools.ietf.org/html/rfc8555#section-7.3.4) for
// the public certificate authority service.
func NewPublicCertificateAuthorityClient(ctx context.Context, opts ...option.ClientOption) (*PublicCertificateAuthorityClient, error) {
	clientOpts := defaultPublicCertificateAuthorityGRPCClientOptions()
	if newPublicCertificateAuthorityClientHook != nil {
		hookOpts, err := newPublicCertificateAuthorityClientHook(ctx, clientHookParams{})
		if err != nil {
			return nil, err
		}
		clientOpts = append(clientOpts, hookOpts...)
	}

	disableDeadlines, err := checkDisableDeadlines()
	if err != nil {
		return nil, err
	}

	connPool, err := gtransport.DialPool(ctx, append(clientOpts, opts...)...)
	if err != nil {
		return nil, err
	}
	client := PublicCertificateAuthorityClient{CallOptions: defaultPublicCertificateAuthorityCallOptions()}

	c := &publicCertificateAuthorityGRPCClient{
		connPool:                         connPool,
		disableDeadlines:                 disableDeadlines,
		publicCertificateAuthorityClient: publiccapb.NewPublicCertificateAuthorityServiceClient(connPool),
		CallOptions:                      &client.CallOptions,
	}
	c.setGoogleClientInfo()

	client.internalClient = c

	return &client, nil
}

// Connection returns a connection to the API service.
//
// Deprecated: Connections are now pooled so this method does not always
// return the same resource.
func (c *publicCertificateAuthorityGRPCClient) Connection() *grpc.ClientConn {
	return c.connPool.Conn()
}

// setGoogleClientInfo sets the name and version of the application in
// the `x-goog-api-client` header passed on each request. Intended for
// use by Google-written clients.
func (c *publicCertificateAuthorityGRPCClient) setGoogleClientInfo(keyval ...string) {
	kv := append([]string{"gl-go", versionGo()}, keyval...)
	kv = append(kv, "gapic", getVersionClient(), "gax", gax.Version, "grpc", grpc.Version)
	c.xGoogMetadata = metadata.Pairs("x-goog-api-client", gax.XGoogHeader(kv...))
}

// Close closes the connection to the API service. The user should invoke this when
// the client is no longer required.
func (c *publicCertificateAuthorityGRPCClient) Close() error {
	return c.connPool.Close()
}

func (c *publicCertificateAuthorityGRPCClient) CreateExternalAccountKey(ctx context.Context, req *publiccapb.CreateExternalAccountKeyRequest, opts ...gax.CallOption) (*publiccapb.ExternalAccountKey, error) {
	if _, ok := ctx.Deadline(); !ok && !c.disableDeadlines {
		cctx, cancel := context.WithTimeout(ctx, 60000*time.Millisecond)
		defer cancel()
		ctx = cctx
	}
	md := metadata.Pairs("x-goog-request-params", fmt.Sprintf("%s=%v", "parent", url.QueryEscape(req.GetParent())))

	ctx = insertMetadata(ctx, c.xGoogMetadata, md)
	opts = append((*c.CallOptions).CreateExternalAccountKey[0:len((*c.CallOptions).CreateExternalAccountKey):len((*c.CallOptions).CreateExternalAccountKey)], opts...)
	var resp *publiccapb.ExternalAccountKey
	err := gax.Invoke(ctx, func(ctx context.Context, settings gax.CallSettings) error {
		var err error
		resp, err = c.publicCertificateAuthorityClient.CreateExternalAccountKey(ctx, req, settings.GRPC...)
		return err
	}, opts...)
	if err != nil {
		return nil, err
	}
	return resp, nil
}
